@@ -15,7 +15,6 @@
 #    limitations under the License.
 
 import abc
-from stevedore import extension
 
 from mistral.openstack.common import log as logging
 
@@ -81,31 +80,3 @@ class Action(object):
             doesn't override this method then the action is synchronous.
         """
         return True
-
-
-class Namespace(object):
-    """Action namespace."""
-
-    def __init__(self, namespace):
-        self.name = namespace.split('.')[-1]
-        self.mgr = extension.ExtensionManager(
-            namespace=namespace,
-            invoke_on_load=False)
-
-    def contains_action_name(self, name):
-        return name in self.mgr.names()
-
-    def get_action_class(self, name):
-        # ExtensionManager has no "get"
-        if self.contains_action_name(name):
-            return self.mgr[name].plugin
-        else:
-            return None
-
-    def __len__(self):
-        # ExtensionManager has no len()
-        return len(self.mgr.names())
-
-    def log(self):
-        for ext in self.mgr:
-            LOG.debug('%s:%s' % (self.name, ext.name))
