@@ -21,28 +21,29 @@ from oslo import messaging
 import six
 from stevedore import driver
 
-# Submoules of mistral.engine will throw NoSuchOptError if configuration
+# Submodules of mistral.engine will throw NoSuchOptError if configuration
 # options required at top level of this  __init__.py are not imported before
 # the submodules are referenced.
 cfg.CONF.import_opt('workflow_trace_log_name', 'mistral.config')
 
 from mistral import context as auth_context
-from mistral.db import api as db_api
-from mistral import dsl_parser as parser
+from mistral.db.v1 import api as db_api
 from mistral.engine import data_flow
 from mistral.engine import retry
 from mistral.engine import states
 from mistral.engine import workflow
 from mistral import exceptions as exc
 from mistral.openstack.common import log as logging
+from mistral.services import action_manager as a_m
+from mistral.workbook import parser as spec_parser
 
 
 LOG = logging.getLogger(__name__)
-WORKFLOW_TRACE = logging.getLogger(cfg.CONF.workflow_trace_log_name)
+WF_TRACE = logging.getLogger(cfg.CONF.workflow_trace_log_name)
 
 
 def get_transport(transport=None):
-    return (transport if transport else messaging.get_transport(cfg.CONF))
+    return transport if transport else messaging.get_transport(cfg.CONF)
 
 
 def get_engine(name, transport):
@@ -64,7 +65,7 @@ class Engine(object):
         self.transport = get_transport(transport)
 
     @abc.abstractmethod
-    def _run_tasks(cls, tasks):
+    def _run_task(self, task_id, action_name, action_params):
         raise NotImplementedError()
 
     def start_workflow_execution(self, cntx, **kwargs):
@@ -83,33 +84,44 @@ class Engine(object):
 
         context = copy.copy(context) if context else {}
 
-        WORKFLOW_TRACE.info("New execution started - [workbook_name = '%s', "
-                            "task_name = '%s']" % (workbook_name, task_name))
+        WF_TRACE.info("New execution started - [workbook_name = '%s', "
+                      "task_name = '%s']" % (workbook_name, task_name))
 
         db_api.start_tx()
 
         # Persist execution and tasks in DB.
         try:
             workbook = self._get_workbook(workbook_name)
-            execution = self._create_execution(workbook_name,
-                                               task_name,
+            execution = self._create_execution(workbook_name, task_name,
                                                context)
 
+            # Create the whole tree of tasks required by target task, including
+            # target task itself.
             tasks = self._create_tasks(
                 workflow.find_workflow_tasks(workbook, task_name),
                 workbook,
-                workbook_name, execution['id']
+                workbook_name, execution.id
             )
 
+            # Create a list of tasks that can be executed immediately (have
+            # their requirements satisfied, or, at that point, rather don't
+            # have them at all) along with the list of tasks that require some
+            # delay before they'll be executed.
             tasks_to_start, delayed_tasks = workflow.find_resolved_tasks(tasks)
 
+            # Populate context with special variables such as `openstack` and
+            # `__execution`.
             self._add_variables_to_data_flow_context(context, execution)
 
-            data_flow.prepare_tasks(tasks_to_start, context)
+            # Update task with new context and params.
+            executables = data_flow.prepare_tasks(tasks_to_start,
+                                                  context,
+                                                  workbook,
+                                                  tasks)
 
             db_api.commit_tx()
         except Exception as e:
-            msg = "Failed to create necessary DB objects: %s" % e
+            msg = "Failed to start workflow execution: %s" % e
             LOG.exception(msg)
             raise exc.EngineException(msg)
         finally:
@@ -118,9 +130,10 @@ class Engine(object):
         for task in delayed_tasks:
             self._schedule_run(workbook, task, context)
 
-        self._run_tasks(tasks_to_start)
+        for task_id, action_name, action_params in executables:
+            self._run_task(task_id, action_name, action_params)
 
-        return execution
+        return execution.to_dict()
 
     def stop_workflow_execution(self, cntx, **kwargs):
         """Stops the workflow execution with the given id.
@@ -133,8 +146,9 @@ class Engine(object):
         """
         execution_id = kwargs.get('execution_id')
 
-        return db_api.execution_update(execution_id,
-                                       {"state": states.STOPPED})
+        return db_api.execution_update(
+            execution_id, {"state": states.STOPPED}
+        ).to_dict()
 
     def convey_task_result(self, cntx, **kwargs):
         """Conveys task result to Mistral Engine.
@@ -163,69 +177,114 @@ class Engine(object):
         try:
             # TODO(rakhmerov): validate state transition
             task = db_api.task_get(task_id)
-            workbook = self._get_workbook(task['workbook_name'])
+            workbook = self._get_workbook(task.workbook_name)
 
-            wf_trace_msg = "Task '%s' [%s -> %s" % \
-                           (task['name'], task['state'], state)
+            if state == states.ERROR:
+                WF_TRACE.info("Task '%s' [%s -> %s]" %
+                              (task.name, task.state, state))
+            else:
+                WF_TRACE.info("Task '%s' [%s -> %s, result = %s]" %
+                              (task.name, task.state, state, result))
 
-            wf_trace_msg += ']' if state == states.ERROR \
-                else ", result = %s]" % result
-            WORKFLOW_TRACE.info(wf_trace_msg)
+            action_name = spec_parser.get_task_spec(task.task_spec)\
+                .get_full_action_name()
+
+            if not a_m.get_action_class(action_name):
+                action = a_m.resolve_adhoc_action_name(workbook, action_name)
+
+                if not action:
+                    msg = 'Unknown action [workbook=%s, action=%s]' % \
+                          (workbook, action_name)
+                    raise exc.ActionException(msg)
+
+                result = a_m.convert_adhoc_action_result(workbook,
+                                                         action_name,
+                                                         result)
 
             task_output = data_flow.get_task_output(task, result)
 
             # Update task state.
-            task, outbound_context = self._update_task(workbook, task, state,
-                                                       task_output)
-
-            execution = db_api.execution_get(task['execution_id'])
+            task, context = self._update_task(workbook, task, state,
+                                              task_output)
 
             self._create_next_tasks(task, workbook)
 
-            # Determine what tasks need to be started.
-            tasks = db_api.tasks_get(workbook_name=task['workbook_name'],
-                                     execution_id=task['execution_id'])
-
-            new_exec_state = self._determine_execution_state(execution, tasks)
-
-            if execution['state'] != new_exec_state:
-                wf_trace_msg = \
-                    "Execution '%s' [%s -> %s]" % \
-                    (execution['id'], execution['state'], new_exec_state)
-                WORKFLOW_TRACE.info(wf_trace_msg)
-
-                execution = \
-                    db_api.execution_update(execution['id'], {
-                        "state": new_exec_state
-                    })
-
-                LOG.info("Changed execution state: %s" % execution)
-
-            tasks_to_start, delayed_tasks = workflow.find_resolved_tasks(tasks)
-
-            self._add_variables_to_data_flow_context(outbound_context,
-                                                     execution)
-
-            data_flow.prepare_tasks(tasks_to_start, outbound_context)
+            # At that point, sqlalchemy tries to flush the changes in task
+            # to the db and, in some cases, hits sqlite database lock
+            # established by another thread of convey_task_results executed
+            # at the same time (for example, as a result of two std.echo
+            # tasks started one after another within the same self._run_task
+            # call). By separating the transaction into two, we creating a
+            # window of opportunity for task changes to be flushed. The
+            # possible ramifications are unclear at the moment and should be
+            # a subject of further review.
 
+            # TODO(rakhmerov): review the possibility to use a single
+            # transaction after switching to the db with better support of
+            # concurrency.
             db_api.commit_tx()
         except Exception as e:
-            msg = "Failed to create necessary DB objects: %s" % e
+            msg = "Failed to save task result: %s" % e
             LOG.exception(msg)
             raise exc.EngineException(msg)
         finally:
             db_api.end_tx()
 
-        if states.is_stopped_or_finished(execution["state"]):
+        db_api.start_tx()
+
+        try:
+            execution = db_api.execution_get(task.execution_id)
+
+            # Determine what tasks need to be started.
+            tasks = db_api.tasks_get(execution_id=execution.id)
+
+            new_exec_state = self._determine_execution_state(execution, tasks)
+
+            if execution.state != new_exec_state:
+                WF_TRACE.info(
+                    "Execution '%s' [%s -> %s]" %
+                    (execution.id, execution.state, new_exec_state)
+                )
+
+                execution = db_api.execution_update(execution.id, {
+                    "state": new_exec_state
+                })
+
+                LOG.info("Changed execution state: %s" % execution)
+
+            # Create a list of tasks that can be executed immediately (have
+            # their requirements satisfied) along with the list of tasks that
+            # require some delay before they'll be executed.
+            tasks_to_start, delayed_tasks = workflow.find_resolved_tasks(tasks)
+
+            # Populate context with special variables such as `openstack` and
+            # `__execution`.
+            self._add_variables_to_data_flow_context(context, execution)
+
+            # Update task with new context and params.
+            executables = data_flow.prepare_tasks(tasks_to_start,
+                                                  context,
+                                                  workbook,
+                                                  tasks)
+
+            db_api.commit_tx()
+        except Exception as e:
+            msg = "Failed to queue next batch of tasks: %s" % e
+            LOG.exception(msg)
+            raise exc.EngineException(msg)
+        finally:
+            db_api.end_tx()
+
+        if states.is_stopped_or_finished(execution.state):
             return task
 
         for task in delayed_tasks:
-            self._schedule_run(workbook, task, outbound_context)
+            self._schedule_run(workbook, task, context)
 
-        if tasks_to_start:
-            self._run_tasks(tasks_to_start)
+        for task_id, action_name, action_params in executables:
+            self._run_task(task_id, action_name, action_params)
 
-        return task
+        return task.to_dict()
 
     def get_workflow_execution_state(self, cntx, **kwargs):
         """Gets the workflow execution state.
@@ -246,7 +305,7 @@ class Engine(object):
                                       "[workbook_name=%s, execution_id=%s]"
                                       % (workbook_name, execution_id))
 
-        return execution["state"]
+        return execution.state
 
     def get_task_state(self, cntx, **kwargs):
         """Gets task state.
@@ -264,7 +323,7 @@ class Engine(object):
         if not task:
             raise exc.EngineException("Task not found.")
 
-        return task["state"]
+        return task.state
 
     @classmethod
     def _create_execution(cls, workbook_name, task_name, context):
@@ -277,7 +336,7 @@ class Engine(object):
 
     @classmethod
     def _add_variables_to_data_flow_context(cls, df_ctx, execution):
-        db_workbook = db_api.workbook_get(execution['workbook_name'])
+        db_workbook = db_api.workbook_get(execution.workbook_name)
 
         data_flow.add_openstack_data_to_context(df_ctx, db_workbook)
         data_flow.add_execution_to_context(df_ctx, execution)
@@ -286,13 +345,13 @@ class Engine(object):
     def _create_next_tasks(cls, task, workbook):
         tasks = workflow.find_tasks_after_completion(task, workbook)
 
-        db_tasks = cls._create_tasks(tasks, workbook, task['workbook_name'],
-                                     task['execution_id'])
+        db_tasks = cls._create_tasks(tasks, workbook, task.workbook_name,
+                                     task.execution_id)
         return workflow.find_resolved_tasks(db_tasks)
 
     @classmethod
     def _create_tasks(cls, task_list, workbook, workbook_name, execution_id):
-        tasks = []
+        tasks = {}
 
         for task in task_list:
             state, task_runtime_context = retry.get_task_runtime(task)
@@ -300,7 +359,8 @@ class Engine(object):
 
             db_task = db_api.task_create(execution_id, {
                 "name": task.name,
-                "requires": task.get_requires(),
+                "requires": [tasks[name].id for name
+                             in task.get_requires()],
                 "task_spec": task.to_dict(),
                 "action_spec": {} if not action_spec
                 else action_spec.to_dict(),
@@ -310,14 +370,15 @@ class Engine(object):
                 "workbook_name": workbook_name
             })
 
-            tasks.append(db_task)
+            tasks[db_task.name] = db_task
 
-        return tasks
+        return tasks.values()
 
     @classmethod
     def _get_workbook(cls, workbook_name):
         wb = db_api.workbook_get(workbook_name)
-        return parser.get_workbook(wb["definition"])
+
+        return spec_parser.get_workbook_spec_from_yaml(wb.definition)
 
     @classmethod
     def _determine_execution_state(cls, execution, tasks):
@@ -327,7 +388,7 @@ class Engine(object):
         if workflow.is_success(tasks) or workflow.is_finished(tasks):
             return states.SUCCESS
 
-        return execution['state']
+        return execution.state
 
     @classmethod
     def _update_task(cls, workbook, task, state, task_output):
@@ -336,8 +397,8 @@ class Engine(object):
         :return: task, outbound_context. task is the updated task and
         computed outbound context.
         """
-        task_spec = workbook.tasks.get(task["name"])
-        task_runtime_context = task["task_runtime_context"]
+        task_spec = workbook.tasks.get(task.name)
+        task_runtime_context = task.task_runtime_context
 
         # Compute the outbound_context, state and exec_flow_context.
         outbound_context = data_flow.get_outbound_context(task, task_output)
@@ -345,14 +406,17 @@ class Engine(object):
             task_spec, state, outbound_context, task_runtime_context)
 
         # Update the task.
-        update_values = {"state": state,
-                         "output": task_output,
-                         "task_runtime_context": task_runtime_context}
+        update_values = {
+            "state": state,
+            "output": task_output,
+            "task_runtime_context": task_runtime_context
+        }
+
         task = db_api.task_update(task["id"], update_values)
 
         return task, outbound_context
 
-    def _schedule_run(cls, workbook, task, outbound_context):
+    def _schedule_run(self, workbook, task, outbound_context):
         """Schedules task to run after the delay defined in the task
         specification. If no delay is specified this method is a no-op.
         """
@@ -373,27 +437,30 @@ class Engine(object):
             db_api.start_tx()
 
             try:
-                execution_id = task['execution_id']
+                execution_id = task.execution_id
                 execution = db_api.execution_get(execution_id)
 
-                # Change state from DELAYED to IDLE to unblock processing.
+                tasks = db_api.tasks_get(execution_id=execution_id)
 
-                WORKFLOW_TRACE.info("Task '%s' [%s -> %s]"
-                                    % (task['name'],
-                                       task['state'], states.IDLE))
+                # Change state from DELAYED to RUNNING.
 
-                db_task = db_api.task_update(task['id'],
-                                             {"state": states.IDLE})
-                task_to_start = [db_task]
-                data_flow.prepare_tasks(task_to_start, outbound_context)
+                WF_TRACE.info("Task '%s' [%s -> %s]" %
+                              (task.name, task.state, states.RUNNING))
+                executables = data_flow.prepare_tasks([task],
+                                                      outbound_context,
+                                                      workbook,
+                                                      tasks)
                 db_api.commit_tx()
             finally:
                 db_api.end_tx()
 
-            if not states.is_stopped_or_finished(execution["state"]):
-                cls._run_tasks(task_to_start)
+            if states.is_stopped_or_finished(execution.state):
+                return
 
-        task_spec = workbook.tasks.get(task['name'])
+            for task_id, action_name, action_params in executables:
+                self._run_task(task_id, action_name, action_params)
+
+        task_spec = workbook.tasks.get(task.name)
         retries, break_on, delay_sec = task_spec.get_retry_parameters()
 
         if delay_sec > 0:
@@ -402,7 +469,7 @@ class Engine(object):
                                  context=auth_context.ctx())
         else:
             LOG.warn("No delay specified for task(id=%s) name=%s. Not "
-                     "scheduling for execution." % (task['id'], task['name']))
+                     "scheduling for execution." % (task.id, task.name))
 
 
 class EngineClient(object):
@@ -429,11 +496,17 @@ class EngineClient(object):
         :param context: Execution context which defines a workflow input
         :return: Workflow execution.
         """
-        cntx = auth_context.ctx()
-        kwargs = {'workbook_name': workbook_name,
-                  'task_name': task_name,
-                  'context': context}
-        return self._client.call(cntx, 'start_workflow_execution', **kwargs)
+        kwargs = {
+            'workbook_name': workbook_name,
+            'task_name': task_name,
+            'context': context
+        }
+
+        return self._client.call(
+            auth_context.ctx(),
+            'start_workflow_execution',
+            **kwargs
+        )
 
     def stop_workflow_execution(self, workbook_name, execution_id):
         """Stops the workflow execution with the given id.
@@ -442,10 +515,16 @@ class EngineClient(object):
         :param execution_id: Workflow execution id.
         :return: Workflow execution.
         """
-        cntx = auth_context.ctx()
-        kwargs = {'workbook_name': workbook_name,
-                  'execution_id': execution_id}
-        return self._client.call(cntx, 'stop_workflow_execution', **kwargs)
+        kwargs = {
+            'workbook_name': workbook_name,
+            'execution_id': execution_id
+        }
+
+        return self._client.call(
+            auth_context.ctx(),
+            'stop_workflow_execution',
+            **kwargs
+        )
 
     def convey_task_result(self, task_id, state, result):
         """Conveys task result to Mistral Engine.
@@ -464,11 +543,17 @@ class EngineClient(object):
         :param result: Task result data.
         :return: Task.
         """
-        cntx = auth_context.ctx()
-        kwargs = {'task_id': task_id,
-                  'state': state,
-                  'result': result}
-        return self._client.call(cntx, 'convey_task_result', **kwargs)
+        kwargs = {
+            'task_id': task_id,
+            'state': state,
+            'result': result
+        }
+
+        return self._client.call(
+            auth_context.ctx(),
+            'convey_task_result',
+            **kwargs
+        )
 
     def get_workflow_execution_state(self, workbook_name, execution_id):
         """Gets the workflow execution state.
@@ -477,11 +562,16 @@ class EngineClient(object):
         :param execution_id: Workflow execution id.
         :return: Current workflow state.
         """
-        cntx = auth_context.ctx()
-        kwargs = {'workbook_name': workbook_name,
-                  'execution_id': execution_id}
+        kwargs = {
+            'workbook_name': workbook_name,
+            'execution_id': execution_id
+        }
+
         return self._client.call(
-            cntx, 'get_workflow_execution_state', **kwargs)
+            auth_context.ctx(),
+            'get_workflow_execution_state',
+            **kwargs
+        )
 
     def get_task_state(self, workbook_name, execution_id, task_id):
         """Gets task state.
@@ -491,8 +581,14 @@ class EngineClient(object):
         :param task_id: Task id.
         :return: Current task state.
         """
-        cntx = auth_context.ctx()
-        kwargs = {'workbook_name': workbook_name,
-                  'executioin_id': execution_id,
-                  'task_id': task_id}
-        return self._client.call(cntx, 'get_task_state', **kwargs)
+        kwargs = {
+            'workbook_name': workbook_name,
+            'executioin_id': execution_id,
+            'task_id': task_id
+        }
+
+        return self._client.call(
+            auth_context.ctx(),
+            'get_task_state',
+            **kwargs
+        )
