@@ -17,6 +17,7 @@
 import logging
 import threading
 
+
 from eventlet import corolocal
 
 # Thread local storage.
@@ -65,13 +66,13 @@ def set_thread_local(var_name, val):
     if val:
         gl_storage = _get_greenlet_local_storage()
         if not gl_storage:
-            gl_storage =\
+            gl_storage = \
                 _th_loc_storage.greenlet_locals[corolocal.get_ident()] = {}
 
         gl_storage[var_name] = val
 
 
-def log_exec(logger, level=logging.INFO):
+def log_exec(logger, level=logging.DEBUG):
     """Decorator for logging function execution.
         By default, target function execution is logged with INFO level.
     """
@@ -93,3 +94,29 @@ def log_exec(logger, level=logging.INFO):
         return _logged
 
     return _decorator
+
+
+def merge_dicts(left, right):
+    """Merges two dictionaries.
+
+    Values of right dictionary recursively get merged into left dictionary.
+    :param left: Left dictionary.
+    :param right: Right dictionary.
+    """
+
+    if left is None:
+        return right
+
+    if right is None:
+        return left
+
+    for k, v in right.iteritems():
+        if k not in left:
+            left[k] = v
+        else:
+            left_v = left[k]
+
+            if isinstance(left_v, dict) and isinstance(v, dict):
+                merge_dicts(left_v, v)
+
+    return left
