@@ -110,7 +110,7 @@ function run_tests {
     if [ "$testropts" = "" ] && [ "$testrargs" = "" ]; then
       # Default to running all tests if specific test is not
       # provided.
-      testrargs="discover ./mistral/tests"
+      testrargs="discover ./mistral/tests/unit"
     fi
     ${wrapper} python -m testtools.run $testropts $testrargs
 
